- test:
    # Timeout for all the TFTF tests to complete.
    timeout:
      minutes: {{ timeout }}
    monitors:
    - name: {{ name }}
      # LAVA looks for a testsuite start string...
      start: 'Booting trusted firmware test framework'
      # ...and a testsuite end string.
      end: 'Exiting tests.'
      # For each test case, LAVA looks for a string which includes the testcase
      # name and result.
      pattern: "(?s)> Executing '(?P<test_case_id>.+?(?='))'(.*)  TEST COMPLETE\\s+(?P<result>(Skipped|Passed|Failed|Crashed))"
      # Teach to LAVA how to interpret the TFTF Tests results.
      fixupdict:
        Passed: pass
        Failed: fail
        Crashed: fail
        Skipped: skip
