- test:
    timeout:
      minutes: {{ timeout }}
    interactive:
    - name: smc91x
      prompts: ["{{ '", "'.join(prompts)}}"]
      script:
      - command: "dmesg"
        name: smc91x
        successes:
        - message: 'virtio_net.{0,4}: probe of virtio1 failed with error -2'

