- test:
    timeout:
      minutes: {{ timeout }}
    definitions:
    - repository: {{ test_definitions }}
{%if device.name != 'ssh-device' %}
      lava-signal: kmsg
{% endif %}
      from: url
      compression: zstd
      path: automated/linux/xfstests/xfstests.yaml
      parameters:
        SKIP_INSTALL: 'true'
        TEST_DEV: '{{ parameters.get("TEST_DEV", "/dev/loop0") }}'
        SCRATCH_DEV: '{{ parameters.get("SCRATCH_DEV", "/dev/loop1") }}'
        TEST_DIR: '{{ parameters.get("TEST_DIR", "/mnt/test") }}'
        SCRATCH_DIR: '{{ parameters.get("SCRATCH_DIR", "/mnt/scratch") }}'
        FILESYSTEM: '{{ test_filesystem }}'
        T_SIZE: '{{ parameters.get("T_SIZE", "5G") }}'
        S_SIZE: '{{ parameters.get("S_SIZE", "8G") }}'
      name: {{ name }}
