- test:
    timeout:
      minutes: {{ timeout }}
    docker:
      image: git.morello-project.org:5050/morello/morello-ci-containers/morello-android-platform-tools
      network_from: "{{ tmpdir.name }}"
    definitions:
    - repository: https://git.morello-project.org/morello/morello-ci-pipelines.git
      from: git
      path: lava/test-definitions/{{ test_def_name }}/testdef.yaml
      name: {{ name }}
{% if parameters %}
      parameters:
{% for key in parameters.keys()|sort %}
        {{ key }}: "{{ parameters.get(key) }}"
{% endfor %}
{% endif %}
