{% extends 'base.yaml.jinja2' %}
{% block body %}

actions:
  deploy:
    connections:
      serial:
    methods:
      ssh:
        options:
{{ ssh_options }}
        # primary connections get this from the device dictionary.
        # secondary connections get this from the lava-multinode protocol support.
        host: "{{ ssh_host|default('') }}"
        port: {{ ssh_port|default(22) }}
        user: "{{ ssh_user|default('root') }}"
        identity_file: "{{ ssh_identity_file }}"

  boot:
    connections:
      serial:
    methods:
      ssh:

{% endblock body %}
