- test:
    timeout:
      minutes: {{ timeout }}
    definitions:
    - repository: {{ test_definitions }}
{%if device.name != 'ssh-device' %}
      lava-signal: kmsg
{% endif %}
      from: url
      compression: zstd
      path: automated/linux/modules/modules.yaml
      name: modules
      parameters:
        MODULES_LIST: '{{ parameters.get("MODULES_LIST", "") }}'
        MODULES_SUBDIRS: '{{ parameters.get("MODULES_SUBDIRS", "") }}'
        MODULE_MODPROBE_NUMBER: '{{ parameters.get("MODULE_MODPROBE_NUMBER", "1") }}'
        SHARD_NUMBER: {{ parameters.get("SHARD_NUMBER", 1) }}
        SHARD_INDEX: {{ parameters.get("SHARD_INDEX", 1) }}
        SKIPLIST: '{{ parameters.get("SKIPLIST", "") }}'

