- test:
    timeout:
      minutes: {{ timeout }}
    definitions:
    - repository: {{ test_definitions }}
{%if device.name != 'ssh-device' %}
      lava-signal: kmsg
{% endif %}
      from: url
      compression: zstd
      path: automated/linux/fwts/fwts.yaml
      parameters:
        SKIP_INSTALL: "true"
        TESTS: "--all"
        SKIP_TESTS: "esrt,esrtdump,cpufreq"
      name: fwts
