- test:
    timeout:
      minutes: {{ timeout }}
    definitions:
    - repository: {{ test_definitions }}
{%if device.name != 'ssh-device' %}
      lava-signal: kmsg
{% endif %}
      from: url
      compression: zstd
      path: automated/linux/libhugetlbfs/libhugetlbfs.yaml
      name: libhugetlbfs
      parameters:
        WORD_SIZE: 64
        SKIP_INSTALL: True
        LIBHUGETLBFS_PATH: '/opt/libhugetlbfs/'
